import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/ocr_services_api_helper.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_receipts_query.dart';

import 'package:dio/dio.dart';

abstract class ReceiptRemoteDataSource {
  Future<Either<Failure, dynamic>> getS3FilePresignedUrl();

  Future<Either<Failure, dynamic>> uploadPresignedUrlFile({
    required List<int> fileBytes,
    required String presignedUrl,
    required Map<String, dynamic> presignedUrlFields,
  });

  Future<Either<Failure, dynamic>> getUserReceipts({
    int? start,
    int? limit,
  });
}

class ReceiptRemoteDataSourceImpl implements ReceiptRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dcouponSDKOptions.dioInterceptors);

  Future<Either<Failure, dynamic>> getS3FilePresignedUrl() async {
    final dioOptions = Options(
      contentType: Headers.jsonContentType,
      headers: UsersApiHelper.buildHeaders(),
    );

    try {
      final response = await dio.get(
        OCRServicesApiHelper.getS3FilePresignedUrl(),
        options: dioOptions,
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  Future<Either<Failure, dynamic>> uploadPresignedUrlFile({
    required List<int> fileBytes,
    required String presignedUrl,
    required Map<String, dynamic> presignedUrlFields,
  }) async {
    final dioOptions = BaseOptions(
      headers: {},
    );

    final freshDio = new Dio(dioOptions);

    try {
      final formData = FormData.fromMap(
        {
          ...presignedUrlFields,
          'file': MultipartFile.fromBytes(
            fileBytes,
          ),
        },
      );

      final response = await freshDio.post(
        presignedUrl,
        data: formData,
      );

      return Right(response.data);
    } on DioError catch (e) {
      throw ServerException(e.message);
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserReceipts({
    int? start,
    int? limit,
  }) async {
    final dioOptions = Options(
      contentType: Headers.jsonContentType,
      headers: UsersApiHelper.buildHeaders(),
    );

    final queryParameters = UserGetReceiptsQuery(
      start: start,
      limit: limit,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getUserReceiptsUrl(),
        queryParameters: queryParameters.toJson(),
        options: dioOptions,
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
